#region Copyright  2009, DevLynx (MIT License)
// This software is licensed under The MIT License:
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
#endregion

namespace AeroGlass
{
    using System;
    using System.Drawing;

    /// <summary>
    /// Arguments passed into the <see cref="AeroColorChanged"/> event.
    /// </summary>
    public class AeroColorChangedEventArgs : EventArgs
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="AeroColorChangedEventArgs"/> class.
        /// </summary>
        /// <param name="aeroColor">Color of the Aero interface.</param>
        /// <param name="isOpaque">if set to <c>true</c> the color is opaque.</param>
        public AeroColorChangedEventArgs(Color aeroColor, bool isOpaque)
        {
            GlassColor = aeroColor;
            IsOpaque = isOpaque;
        }

        /// <summary>
        /// Gets or sets the Aero color.
        /// </summary>
        /// <value>The Aero color.</value>
        public Color GlassColor { get; set; }

        /// <summary>
        /// Gets or sets a value indicating whether the Aero color is opaque.
        /// </summary>
        /// <value><c>true</c> if the Aero color is opaque; otherwise, <c>false</c>.</value>
        public bool IsOpaque { get; set; }
    }
}
