#region Copyright  2009, DevLynx (MIT License)
// This software is licensed under The MIT License:
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
#endregion

namespace AeroGlass
{
    using System.Drawing;

    /// <summary>
    /// Properties associated with a control on a glass enabled form.
    /// </summary>
    public class AeroGlassControlProperties
    {

        /// <summary>
        /// The default text glow size for drawing text into the class area.
        /// </summary>
        public const int DefaultGlassTextGlowSize = 10;


        /// <summary>
        /// Initializes a new instance of the <see cref="AeroGlassControlProperties"/> class.
        /// </summary>
        public AeroGlassControlProperties()
        {
            OnGlass = false;
            GlassTextGlowSize = DefaultGlassTextGlowSize;
            BackColor = Color.Transparent;
        }


        /// <summary>
        /// Gets or sets the background color of the control. This is used to ensure that 
        /// the proper background color is set when the glass cannot be enabled.
        /// </summary>
        /// <value>The background color.</value>
        public Color BackColor { get; set; }

        /// <summary>
        /// Gets or sets the glass text glow size. The larger the number the more 
        /// diffused the background glow behind the text.
        /// </summary>
        /// <value>The glass text glow size.</value>
        public int GlassTextGlowSize { get; set; }

        /// <summary>
        /// Gets or sets a value indicating whether the control is located on the glass.
        /// </summary>
        /// <value><c>true</c> if the control is considered to be on the glass; otherwise, <c>false</c>.</value>
        public bool OnGlass { get; set; }
    }
}
