#region Copyright  2009, DevLynx (MIT License)
// This software is licensed under The MIT License:
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
//
// Algorithms and inspiration from:
//   http://channel9.msdn.com/playground/Sandbox/201158/
//   http://www.codeproject.com/KB/vista/textonglass.aspx
#endregion

namespace AeroGlass
{
    using System;
    using System.Drawing;
    using System.Runtime.InteropServices;
    using System.Windows.Forms;

    /// <summary>
    /// Static utility class that holds the unmanaged Windows methods for the Aero glass effect.
    /// </summary>
    internal static class InteropServices
    {

        internal const int BI_RGB = 0;
        internal const int DIB_RGB_COLORS = 0;
        internal const int DT_CENTER = 0x00000001;
        internal const int DT_NOPREFIX = 0x00000800;
        internal const int DT_SINGLELINE = 0x00000020;
        internal const int DT_VCENTER = 0x00000004;
        internal const int DTT_COMPOSITED = (int)(1UL << 13);
        internal const int DTT_GLOWSIZE = (int)(1UL << 11);
        internal const int HTCAPTION = 2;
        internal const int HTCLIENT = 1;
        internal const int SRCCOPY = 0x00CC0020;
        //internal const int WM_DWMNCRENDERINGCHANGED = 0x031F;
        //internal const int WM_DWMWINDOWMAXIMIZEDCHANGE = 0x0321;
        internal const int WM_DWMCOLORIZATIONCOLORCHANGED = 0x0320;
        internal const int WM_DWMCOMPOSITIONCHANGED = 0x031E;
        internal const int WM_NCHITTEST = 0x84;


        [DllImport("gdi32.dll", ExactSpelling = true, SetLastError = true)]
        internal static extern bool BitBlt(IntPtr hdc, int nXDest, int nYDest, int nWidth, int nHeight, IntPtr hdcSrc, int nXSrc, int nYSrc, uint dwRop);

        [DllImport("gdi32.dll", ExactSpelling = true, SetLastError = true)]
        internal static extern IntPtr CreateCompatibleDC(IntPtr hDC);

        [DllImport("gdi32.dll", ExactSpelling = true, SetLastError = true)]
        internal static extern IntPtr CreateDIBSection(IntPtr hdc, ref BITMAPINFO pbmi, uint iUsage, int ppvBits, IntPtr hSection, uint dwOffset);

        [DllImport("gdi32.dll", ExactSpelling = true, SetLastError = true)]
        internal static extern bool DeleteDC(IntPtr hdc);

        [DllImport("gdi32.dll", ExactSpelling = true, SetLastError = true)]
        internal static extern bool DeleteObject(IntPtr hObject);

        [DllImport("UxTheme.dll", ExactSpelling = true, SetLastError = true)]
        internal static extern int DrawThemeText(IntPtr hTheme, IntPtr hdc, int iPartId, int iStateId, string text, int iCharCount, int dwFlags1, int dwFlags2, ref RECT pRect);

        [DllImport("UxTheme.dll", ExactSpelling = true, SetLastError = true, CharSet = CharSet.Unicode)]
        internal static extern int DrawThemeTextEx(IntPtr hTheme, IntPtr hdc, int iPartId, int iStateId, string text, int iCharCount, int dwFlags, ref RECT pRect, ref DTTOPTS pOptions);

        internal static bool DwmCompositionChanged(Message message)
        {
            return message.Msg == WM_DWMCOMPOSITIONCHANGED;
        }

        internal static bool DwmCompositionColorChanged(Message message)
        {
            return message.Msg == WM_DWMCOLORIZATIONCOLORCHANGED;
        }

        [DllImport("dwmapi.dll", PreserveSig = false, ExactSpelling = true, SetLastError = true)]
        internal static extern void DwmExtendFrameIntoClientArea(IntPtr hwnd, ref MARGINS margins);

        [DllImport("dwmapi.dll", PreserveSig = false, ExactSpelling = true, SetLastError = true)]
        internal static extern void DwmGetColorizationColor(out Int32 ColorizationColor, out bool IsOpaque);

        [DllImport("dwmapi.dll", PreserveSig = false, ExactSpelling = true, SetLastError = true)]
        internal static extern bool DwmIsCompositionEnabled();

         [DllImport("user32.dll", ExactSpelling = true, SetLastError = true)]
        internal static extern IntPtr GetDC(IntPtr hdc);

        [DllImport("user32.dll", ExactSpelling = true, SetLastError = true)]
        internal static extern int ReleaseDC(IntPtr hdc, int state);

        [DllImport("gdi32.dll", ExactSpelling = true, SetLastError = true)]
        internal static extern int SaveDC(IntPtr hdc);

        [DllImport("gdi32.dll", ExactSpelling = true)]
        internal static extern IntPtr SelectObject(IntPtr hDC, IntPtr hObject);


        /// <summary>
        /// Struct that specifies the margin of the glass.
        /// </summary>
        internal struct MARGINS
        {

            private int _00Left;
            private int _01Right;
            private int _02Top;
            private int _03Bottom;


            internal MARGINS(Padding padding)
            {
                _00Left = (int)padding.Left;
                _01Right = (int)padding.Right;
                _02Top = (int)padding.Top;
                _03Bottom = (int)padding.Bottom;
            }
        }
        internal struct RECT
        {

            internal int _00Left;
            internal int _01Top;
            internal int _02Right;
            internal int _03Bottom;


            internal RECT(Rectangle rectangle)
            {
                _00Left = rectangle.Left;
                _01Top = rectangle.Top;
                _02Right = rectangle.Right;
                _03Bottom = rectangle.Bottom;
            }
        }
        internal struct BITMAPINFOHEADER
        {

            public int _00biSize;
            public int _01biWidth;
            public int _02biHeight;
            public short _03biPlanes;
            public short _04biBitCount;
            public int _05biCompression;
            public int _06biSizeImage;
            public int _07biXPelsPerMeter;
            public int _08biYPelsPerMeter;
            public int _09biClrUsed;
            public int _10biClrImportant;
        }
        internal struct RGBQUAD
        {

            public byte _00rgbBlue;
            public byte _01rgbGreen;
            public byte _02rgbRed;
            public byte _03rgbReserved;
        }
        internal struct BITMAPINFO
        {

            public BITMAPINFOHEADER _01bmiHeader;
            public RGBQUAD _02bmiColors;


            internal BITMAPINFO(int Width, int Height)
            {
                _01bmiHeader._00biSize = Marshal.SizeOf(typeof(InteropServices.BITMAPINFOHEADER));
                _01bmiHeader._01biWidth = Width;
                _01bmiHeader._02biHeight = Height; 
                _01bmiHeader._03biPlanes = 1;
                _01bmiHeader._04biBitCount = 32;
                _01bmiHeader._05biCompression = InteropServices.BI_RGB;
                _01bmiHeader._06biSizeImage = default(int);
                _01bmiHeader._07biXPelsPerMeter = default(int);
                _01bmiHeader._08biYPelsPerMeter = default(int);
                _01bmiHeader._09biClrUsed = default(int);
                _01bmiHeader._10biClrImportant = default(int);

                _02bmiColors._00rgbBlue = default(byte);
                _02bmiColors._01rgbGreen = default(byte);
                _02bmiColors._02rgbRed = default(byte);
                _02bmiColors._03rgbReserved = default(byte);
            }
        }
        internal struct POINTAPI
        {

            public int x;
            public int y;


            internal POINTAPI(int x, int y)
            {
                this.x = default(int);
                this.y = default(int);
            }
        }
        internal struct DTTOPTS
        {

            internal uint _01dwSize;
            internal uint _02dwFlags;
            internal uint _03crText;
            internal uint _04crBorder;
            internal uint _05crShadow;
            internal int _06iTextShadowType;
            internal POINTAPI _07ptShadowOffset;
            internal int _08iBorderSize;
            internal int _09iFontPropId;
            internal int _10iColorPropId;
            internal int _11iStateId;
            internal int _12fApplyOverlay;
            internal int _13iGlowSize;
            internal IntPtr _14pfnDrawTextCallback;
            internal int _15lParam;


            internal DTTOPTS(int glowSize)
            {
                _01dwSize = (uint)Marshal.SizeOf(typeof(InteropServices.DTTOPTS));
                _02dwFlags = InteropServices.DTT_COMPOSITED | InteropServices.DTT_GLOWSIZE;
                _03crText = default(uint);
                _04crBorder = default(uint);
                _05crShadow = default(uint);
                _06iTextShadowType = default(int);
                _07ptShadowOffset = new POINTAPI();
                _08iBorderSize = default(int);
                _09iFontPropId = default(int);
                _10iColorPropId = default(int);
                _11iStateId = default(int);
                _12fApplyOverlay = default(int);
                _13iGlowSize = glowSize;
                _14pfnDrawTextCallback = IntPtr.Zero;
                _15lParam = default(int);
            }
        }
    }
}
